import { Interface } from '@ethersproject/abi';
import { Signer } from '@ethersproject/abstract-signer';
import { TransactionResponse } from '@ethersproject/providers';
import { BigintIsh, Currency, CurrencyAmount, Percent, TradeType } from '@uniswap/sdk-core';
import { BestTradeOptions, Trade } from './entities/trade';
import { PermitOptions } from './selfPermit';
import { MethodParameters } from './utils/calldata';
import { FeeOptions } from './payments';
import { Pool } from './entities/pool';
import { Route } from './entities/route';
/**
 * Options for producing the arguments to send calls to the router.
 */
export interface SwapOptions {
    /**
     * How much the execution price is allowed to move unfavorably from the trade execution price.
     */
    slippageTolerance: Percent;
    /**
     * The account that should receive the output.
     */
    recipient: string;
    /**
     * When the transaction expires, in epoch seconds.
     */
    deadline: BigintIsh;
    /**
     * The optional permit parameters for spending the input.
     */
    inputTokenPermit?: PermitOptions;
    /**
     * The optional price limit for the trade.
     */
    sqrtPriceLimitX96?: BigintIsh;
    /**
     * Optional information for taking a fee on output.
     */
    fee?: FeeOptions;
}
/**
 * Represents the Uniswap V3 SwapRouter, and has static methods for helping execute trades.
 */
export declare abstract class SwapRouter {
    static INTERFACE: Interface;
    /**
     * Cannot be constructed.
     */
    private constructor();
    /**
     * Produces the on-chain method name to call and the hex encoded parameters to pass as arguments for a given trade.
     * @param trade to produce call parameters for
     * @param options options for the call parameters
     */
    static swapCallParameters(trades: Trade<Currency, Currency, TradeType> | Trade<Currency, Currency, TradeType>[], options: SwapOptions): MethodParameters;
    /**
     * Utility function that creates calldata for Trades with swapCallParameters and directly calls the function using a given Signer.
     * @param trades to produce call parameters for
     * @param options for the call parameters
     * @param signer with provider capabilities to sign the transaction
     */
    static executeTrade(trades: Trade<Currency, Currency, TradeType> | Trade<Currency, Currency, TradeType>[], options: SwapOptions | undefined, signer: Signer): Promise<TransactionResponse>;
    /**
     * Executes a swap on a Pool. Creates a Route and uses swapUsingRoutes internally.
     * Fetches the input or output amount from the SwapQuoter.
     * @template TInput The input token, either Ether or an ERC-20
     * @template TOutput The output token, either Ether or an ERC-20
     * @param pool to execute the swap on.
     * @param amount The Input amount for EXACT_IN trades, the Output amount for EXACT_OUT trades
     * @param tradeType of the swap, either EXACT_IN or EXACT_OUT
     * @param swapOptions Optional SwapOptions that define deadline, recipient and slippage of the trade
     * @param signer with provider capabilities to sign the transaction
     */
    static executeQuotedSwapOnPool<TInput extends Currency, TOutput extends Currency, TTradeType extends TradeType>(pool: Pool, amount: TTradeType extends TradeType.EXACT_INPUT ? CurrencyAmount<TInput> : CurrencyAmount<TOutput>, tradeType: TradeType, swapOptions: SwapOptions | undefined, signer: Signer): Promise<TransactionResponse>;
    /**
     * Executes trades along a route. Uses the SwapQuoter to quote the amounts.
     * @template TInput The input token, either Ether or an ERC-20
     * @template TOutput The output token, either Ether or an ERC-20
     * @param route along which the trade is created
     * @param amount The Input amount for EXACT_IN trades, the Output amount for EXACT_OUT trades
     * @param tradeType of the swap, either EXACT_IN or EXACT_OUT
     * @param swapOptions Optional SwapOptions that define deadline, recipient and slippage of the trade
     * @param signer with provider capabilities to sign the transaction
     */
    static executeQuotedSwapFromRoute<TInput extends Currency, TOutput extends Currency, TTradeType extends TradeType>(route: Route<TInput, TOutput>, amount: TTradeType extends TradeType.EXACT_INPUT ? CurrencyAmount<TInput> : CurrencyAmount<TOutput>, tradeType: TradeType, swapOptions: SwapOptions | undefined, signer: Signer): Promise<TransactionResponse>;
    /**
     * Given a list of Pools, and an amount in or out, finds the best trade containing the pools and
     * executes the trade directly.
     * @param pools over which the trade can be made
     * @param amount The Input amount for EXACT_IN trades, the Output amount for EXACT_OUT trades
     * @param currencyIn for EXACT_OUT trades
     * @param currencyOut for EXACT_IN trades
     * @param tradeType of the Trade, EXACT_IN or EXACT_OUT
     * @param bestTradeOptions Optional, defines the max number of trades used and max pools (hops) per route
     * @param swapOptions Optional Swap Options for the trade execution
     * @param signer with provider capabilities to sign the transaction
     */
    static executeBestSimulatedSwapOnPools<TInput extends Currency, TOutput extends Currency, TTradeType extends TradeType>(pools: Pool[], amount: TTradeType extends TradeType.EXACT_INPUT ? CurrencyAmount<TInput> : CurrencyAmount<TOutput>, currencyIn: TTradeType extends TradeType.EXACT_INPUT ? undefined | TInput : TInput, currencyOut: TTradeType extends TradeType.EXACT_OUTPUT ? undefined | TOutput : TOutput, tradeType: TradeType, bestTradeOptions: BestTradeOptions | undefined, swapOptions: SwapOptions | undefined, signer: Signer): Promise<TransactionResponse>;
    /**
     * Creates a Trade from a Route and executes it using a given Signer.
     * @param route The Route through which the trade is made
     * @param amount The input amount for EXACT_IN trades, the output amount for EXACT_OUT trades
     * @param tradeType TradeType, EXACT_IN or EXACT_OUT
     * @param swapOptions Optional Swap Options for the trade execution
     * @param signer with provider capabilities to sign the Transaction
     */
    static executeSimulatedSwapFromRoute<TInput extends Currency, TOutput extends Currency, TTradeType extends TradeType>(route: Route<TInput, TOutput>, amount: TTradeType extends TradeType.EXACT_INPUT ? CurrencyAmount<TInput> : CurrencyAmount<TOutput>, tradeType: TradeType, swapOptions: SwapOptions | undefined, signer: Signer): Promise<TransactionResponse>;
}
