import { Interface } from '@ethersproject/abi';
import { CurrencyAmount, Percent, TradeType, validateAndParseAddress, SUPPORTED_CHAINS, CHAIN_TO_ADDRESSES_MAP } from '@uniswap/sdk-core';
import invariant from 'tiny-invariant';
import { Trade } from './entities/trade';
import { ADDRESS_ZERO } from './constants';
import { SelfPermit } from './selfPermit';
import { approveTokenTransfer, encodeRouteToPath, getAllowance } from './utils';
import { toHex } from './utils/calldata';
import ISwapRouter from '@uniswap/v3-periphery/artifacts/contracts/SwapRouter.sol/SwapRouter.json';
import { Multicall } from './multicall';
import { Payments } from './payments';
import { Route } from './entities/route';
import { SwapQuoter } from './quoter';
import { fetchTickDataForAllPoolsInRoute } from './utils/fetchTicksForRoute';
/**
 * Represents the Uniswap V3 SwapRouter, and has static methods for helping execute trades.
 */
export class SwapRouter {
    /**
     * Cannot be constructed.
     */
    constructor() { }
    /**
     * Produces the on-chain method name to call and the hex encoded parameters to pass as arguments for a given trade.
     * @param trade to produce call parameters for
     * @param options options for the call parameters
     */
    static swapCallParameters(trades, options) {
        if (!Array.isArray(trades)) {
            trades = [trades];
        }
        const sampleTrade = trades[0];
        const tokenIn = sampleTrade.inputAmount.currency.wrapped;
        const tokenOut = sampleTrade.outputAmount.currency.wrapped;
        // All trades should have the same starting and ending token.
        invariant(trades.every((trade) => trade.inputAmount.currency.wrapped.equals(tokenIn)), 'TOKEN_IN_DIFF');
        invariant(trades.every((trade) => trade.outputAmount.currency.wrapped.equals(tokenOut)), 'TOKEN_OUT_DIFF');
        const calldatas = [];
        const ZERO_IN = CurrencyAmount.fromRawAmount(trades[0].inputAmount.currency, 0);
        const ZERO_OUT = CurrencyAmount.fromRawAmount(trades[0].outputAmount.currency, 0);
        const totalAmountOut = trades.reduce((sum, trade) => sum.add(trade.minimumAmountOut(options.slippageTolerance)), ZERO_OUT);
        // flag for whether a refund needs to happen
        const mustRefund = sampleTrade.inputAmount.currency.isNative && sampleTrade.tradeType === TradeType.EXACT_OUTPUT;
        const inputIsNative = sampleTrade.inputAmount.currency.isNative;
        // flags for whether funds should be send first to the router
        const outputIsNative = sampleTrade.outputAmount.currency.isNative;
        const routerMustCustody = outputIsNative || !!options.fee;
        const totalValue = inputIsNative
            ? trades.reduce((sum, trade) => sum.add(trade.maximumAmountIn(options.slippageTolerance)), ZERO_IN)
            : ZERO_IN;
        // encode permit if necessary
        if (options.inputTokenPermit) {
            invariant(sampleTrade.inputAmount.currency.isToken, 'NON_TOKEN_PERMIT');
            calldatas.push(SelfPermit.encodePermit(sampleTrade.inputAmount.currency, options.inputTokenPermit));
        }
        const recipient = validateAndParseAddress(options.recipient);
        const deadline = toHex(options.deadline);
        for (const trade of trades) {
            for (const { route, inputAmount, outputAmount } of trade.swaps) {
                const amountIn = toHex(trade.maximumAmountIn(options.slippageTolerance, inputAmount).quotient);
                const amountOut = toHex(trade.minimumAmountOut(options.slippageTolerance, outputAmount).quotient);
                // flag for whether the trade is single hop or not
                const singleHop = route.pools.length === 1;
                if (singleHop) {
                    if (trade.tradeType === TradeType.EXACT_INPUT) {
                        const exactInputSingleParams = {
                            tokenIn: route.tokenPath[0].address,
                            tokenOut: route.tokenPath[1].address,
                            fee: route.pools[0].fee,
                            recipient: routerMustCustody ? ADDRESS_ZERO : recipient,
                            deadline,
                            amountIn,
                            amountOutMinimum: amountOut,
                            sqrtPriceLimitX96: toHex(options.sqrtPriceLimitX96 ?? 0),
                        };
                        calldatas.push(SwapRouter.INTERFACE.encodeFunctionData('exactInputSingle', [exactInputSingleParams]));
                    }
                    else {
                        const exactOutputSingleParams = {
                            tokenIn: route.tokenPath[0].address,
                            tokenOut: route.tokenPath[1].address,
                            fee: route.pools[0].fee,
                            recipient: routerMustCustody ? ADDRESS_ZERO : recipient,
                            deadline,
                            amountOut,
                            amountInMaximum: amountIn,
                            sqrtPriceLimitX96: toHex(options.sqrtPriceLimitX96 ?? 0),
                        };
                        calldatas.push(SwapRouter.INTERFACE.encodeFunctionData('exactOutputSingle', [exactOutputSingleParams]));
                    }
                }
                else {
                    invariant(options.sqrtPriceLimitX96 === undefined, 'MULTIHOP_PRICE_LIMIT');
                    const path = encodeRouteToPath(route, trade.tradeType === TradeType.EXACT_OUTPUT);
                    if (trade.tradeType === TradeType.EXACT_INPUT) {
                        const exactInputParams = {
                            path,
                            recipient: routerMustCustody ? ADDRESS_ZERO : recipient,
                            deadline,
                            amountIn,
                            amountOutMinimum: amountOut,
                        };
                        calldatas.push(SwapRouter.INTERFACE.encodeFunctionData('exactInput', [exactInputParams]));
                    }
                    else {
                        const exactOutputParams = {
                            path,
                            recipient: routerMustCustody ? ADDRESS_ZERO : recipient,
                            deadline,
                            amountOut,
                            amountInMaximum: amountIn,
                        };
                        calldatas.push(SwapRouter.INTERFACE.encodeFunctionData('exactOutput', [exactOutputParams]));
                    }
                }
            }
        }
        // unwrap
        if (routerMustCustody) {
            if (!!options.fee) {
                if (outputIsNative) {
                    calldatas.push(Payments.encodeUnwrapWETH9(totalAmountOut.quotient, recipient, options.fee));
                }
                else {
                    calldatas.push(Payments.encodeSweepToken(sampleTrade.outputAmount.currency.wrapped, totalAmountOut.quotient, recipient, options.fee));
                }
            }
            else {
                calldatas.push(Payments.encodeUnwrapWETH9(totalAmountOut.quotient, recipient));
            }
        }
        // refund
        if (mustRefund) {
            calldatas.push(Payments.encodeRefundETH());
        }
        return {
            calldata: Multicall.encodeMulticall(calldatas),
            value: toHex(totalValue.quotient),
        };
    }
    /**
     * Utility function that creates calldata for Trades with swapCallParameters and directly calls the function using a given Signer.
     * @param trades to produce call parameters for
     * @param options for the call parameters
     * @param signer with provider capabilities to sign the transaction
     */
    static async executeTrade(trades, options, signer) {
        if (options === undefined) {
            options = {
                slippageTolerance: new Percent(50, 10000),
                deadline: Math.floor(Date.now() / 1000) + 60 * 5,
                recipient: await signer.getAddress(),
            };
        }
        const methodParameters = this.swapCallParameters(trades, options);
        if (!Array.isArray(trades)) {
            trades = [trades];
        }
        const firstTrade = trades[0];
        const tokenIn = firstTrade.inputAmount.currency.wrapped;
        const chainId = tokenIn.chainId;
        const chain = SUPPORTED_CHAINS[chainId];
        invariant(chain !== undefined, 'Unsupported Chain');
        const contractAddresses = CHAIN_TO_ADDRESSES_MAP[chain];
        const routerAddress = contractAddresses.swapRouter02Address;
        invariant(routerAddress !== undefined, 'Router not deployed on requested Chain');
        const signerAddress = await signer.getAddress();
        const inputIsNative = firstTrade.inputAmount.currency.isNative;
        if (inputIsNative === false && !options.inputTokenPermit) {
            const provider = signer.provider;
            invariant(provider !== undefined, 'No provider');
            const allowance = await getAllowance(routerAddress, tokenIn.address, signerAddress, provider);
            const maxAmountIn = BigInt(firstTrade.maximumAmountIn(new Percent(0, 1)).toFixed(0));
            if (allowance < maxAmountIn) {
                await approveTokenTransfer(routerAddress, tokenIn.address, maxAmountIn, signer);
            }
        }
        const tx = {
            data: methodParameters.calldata,
            to: routerAddress,
            value: methodParameters.value,
            from: signerAddress
        };
        return signer.sendTransaction(tx);
    }
    /**
     * Executes a swap on a Pool. Creates a Route and uses swapUsingRoutes internally.
     * Fetches the input or output amount from the SwapQuoter.
     * @template TInput The input token, either Ether or an ERC-20
     * @template TOutput The output token, either Ether or an ERC-20
     * @param pool to execute the swap on.
     * @param amount The Input amount for EXACT_IN trades, the Output amount for EXACT_OUT trades
     * @param tradeType of the swap, either EXACT_IN or EXACT_OUT
     * @param swapOptions Optional SwapOptions that define deadline, recipient and slippage of the trade
     * @param signer with provider capabilities to sign the transaction
     */
    static async executeQuotedSwapOnPool(pool, amount, tradeType, swapOptions, signer) {
        const secondCurrency = amount.currency.equals(pool.token0) ? pool.token1 : pool.token0;
        invariant((secondCurrency.equals(pool.token0) && amount.currency.equals(pool.token1)) || (secondCurrency.equals(pool.token1) && amount.currency.equals(pool.token0)), 'CurrencyAmount not matching Pool');
        const { inputToken, outputToken } = tradeType === TradeType.EXACT_INPUT ?
            { inputToken: amount.currency, outputToken: secondCurrency } :
            { inputToken: secondCurrency, outputToken: amount.currency };
        const tradeRoute = new Route([pool], inputToken, outputToken);
        return this.executeQuotedSwapFromRoute(tradeRoute, amount, tradeType, swapOptions, signer);
    }
    /**
     * Executes trades along a route. Uses the SwapQuoter to quote the amounts.
     * @template TInput The input token, either Ether or an ERC-20
     * @template TOutput The output token, either Ether or an ERC-20
     * @param route along which the trade is created
     * @param amount The Input amount for EXACT_IN trades, the Output amount for EXACT_OUT trades
     * @param tradeType of the swap, either EXACT_IN or EXACT_OUT
     * @param swapOptions Optional SwapOptions that define deadline, recipient and slippage of the trade
     * @param signer with provider capabilities to sign the transaction
     */
    static async executeQuotedSwapFromRoute(route, amount, tradeType, swapOptions, signer) {
        let inputAmount;
        let outputAmount;
        let provider = signer.provider;
        invariant(provider !== undefined, 'Signer needs to have a provider');
        if (tradeType === TradeType.EXACT_INPUT) {
            inputAmount = amount;
            outputAmount = await SwapQuoter.callQuoter(route, amount, tradeType, provider);
        }
        else {
            outputAmount = amount;
            inputAmount = await SwapQuoter.callQuoter(route, amount, tradeType, provider);
        }
        let trade = Trade.createUncheckedTrade({ route, inputAmount, outputAmount, tradeType });
        return this.executeTrade(trade, swapOptions, signer);
    }
    /**
     * Given a list of Pools, and an amount in or out, finds the best trade containing the pools and
     * executes the trade directly.
     * @param pools over which the trade can be made
     * @param amount The Input amount for EXACT_IN trades, the Output amount for EXACT_OUT trades
     * @param currencyIn for EXACT_OUT trades
     * @param currencyOut for EXACT_IN trades
     * @param tradeType of the Trade, EXACT_IN or EXACT_OUT
     * @param bestTradeOptions Optional, defines the max number of trades used and max pools (hops) per route
     * @param swapOptions Optional Swap Options for the trade execution
     * @param signer with provider capabilities to sign the transaction
     */
    static async executeBestSimulatedSwapOnPools(pools, amount, currencyIn, currencyOut, tradeType, bestTradeOptions, swapOptions, signer) {
        if (bestTradeOptions === undefined)
            bestTradeOptions = { maxNumResults: 3, maxHops: 3 };
        const provider = signer.provider;
        invariant(provider !== undefined, 'Signer is not connected to a network.');
        // TODO: Ensure the blocknumber is consistent between these calls
        let promises = [];
        for (let pool of pools) {
            promises.push(pool.initializeTicks());
        }
        await Promise.all(promises);
        let trades;
        if (tradeType === TradeType.EXACT_INPUT) {
            trades = await Trade.bestTradeExactIn(pools, amount, currencyOut, bestTradeOptions);
        }
        else {
            trades = await Trade.bestTradeExactOut(pools, currencyIn, amount);
        }
        invariant(trades.length > 0, 'Could not find a trade route on given Pools');
        return this.executeTrade(trades[0], swapOptions, signer);
    }
    /**
     * Creates a Trade from a Route and executes it using a given Signer.
     * @param route The Route through which the trade is made
     * @param amount The input amount for EXACT_IN trades, the output amount for EXACT_OUT trades
     * @param tradeType TradeType, EXACT_IN or EXACT_OUT
     * @param swapOptions Optional Swap Options for the trade execution
     * @param signer with provider capabilities to sign the Transaction
     */
    static async executeSimulatedSwapFromRoute(route, amount, tradeType, swapOptions, signer) {
        const provider = signer.provider;
        invariant(provider !== undefined, 'Signer has no network connection');
        await fetchTickDataForAllPoolsInRoute(route, provider);
        let trade = await Trade.fromRoute(route, amount, tradeType);
        return this.executeTrade(trade, swapOptions, signer);
    }
}
SwapRouter.INTERFACE = new Interface(ISwapRouter.abi);
//# sourceMappingURL=swapRouter.js.map